package de.hhu.ba.yoshikoWrapper.core;

import java.util.List;

import org.cytoscape.model.CyColumn;
import org.cytoscape.model.CyEdge;
import org.cytoscape.model.CyNetwork;
import org.cytoscape.model.CyRow;
import org.slf4j.Logger;

import de.hhu.ba.yoshikoWrapper.logging.YoshikoLogger;
import de.hhu.ba.yoshikoWrapper.swig.LibraryInterface;
import de.hhu.ba.yoshikoWrapper.swig.SWIGTYPE_p_yskInput__LibraryInput;

public class NetworkParser {
	
	//Symbolic Links
	private static Logger logger = YoshikoLogger.getInstance().getLogger();

	public static SWIGTYPE_p_yskInput__LibraryInput parseNetwork(
			CyNetwork net,
			NodeMap nodeMap,
			CyColumn weightColumn,
			CyColumn permanentColumn,
			CyColumn forbiddenColumn,
			double deletionCostDefault
			) 
	{
		//Create an empty instance / fetch C++ object pointer
		SWIGTYPE_p_yskInput__LibraryInput generatedInput = LibraryInterface.new_LibraryInput();
		if (net != null){
			//Create an empty full graph with a given size
			LibraryInterface.LibraryInput_setSize(generatedInput,  net.getNodeCount());
			//Fetch edges
			List<CyEdge> edges = net.getEdgeList();
			

			
			//Loop over edges
			for (CyEdge e : edges) {
				
				//Parse editing costs
				double weight = deletionCostDefault;
				
				//Fetch entry and check if it exists
				CyRow edgeEntry = net.getRow(e);
				
				//Check if there is a weight column defined, else skip
				if (weightColumn != null){
					//Check if the column contains an entry for the respective edge
					//It is possible, that there are missing entries
					if (edgeEntry.get(weightColumn.getName(), weightColumn.getType()) != null){
						if (weightColumn.getType() == Integer.class) {
							weight = edgeEntry.get(weightColumn.getName(), Integer.class);
						}
						else if (weightColumn.getType() == Double.class) {
							weight = edgeEntry.get(weightColumn.getName(), Double.class);
						}
					}
				}

				//Parse Forbidden/Permanent markers
				boolean forbidden = false;
				boolean permanent = false;
				if (permanentColumn != null) {
					//Additional check as it is not required to have a value in every row
					if (edgeEntry.get(permanentColumn.getName(), Boolean.class) != null) {
						permanent =  (boolean)edgeEntry.get(permanentColumn.getName(), Boolean.class);
					}
				}
				if (forbiddenColumn != null) {
					//Additional check as it is not required to have a value in every row
					if (edgeEntry.get(forbiddenColumn.getName(), Boolean.class) != null) {
						forbidden =  (boolean)edgeEntry.get(forbiddenColumn.getName(), Boolean.class);
					}
				}

				System.out.println("Found Edge: "+edgeEntry.get("name", String.class)+ " with weight:"+weight);

				logger.debug("Found Edge: "+edgeEntry.get("name", String.class)+ " with weight:"+weight);
				
				LibraryInterface.LibraryInput_addEdge(
						generatedInput,
						nodeMap.get(e.getSource()),
						nodeMap.get(e.getTarget()),
						weight,
						permanent,
						forbidden
						);
				
			}
		}
		return generatedInput;
	}

}
